from .my_odt_checker_anchor import MyOdtCheckerAnchor
from .setting_extend_diagram import SettingExtendDiagram
from .setting_extend_set import SettingExtendSet
from .setting_add_images import SettingAddImages
from .odt_apply_settings import ODTApplySettings
from .setting_if_block import SettingIfBlock
